@@ -3,6 +3,10 @@ use strict;
 use diagnostics;
 use 5.010000;
 use XSLoader ();
+#
+# Internally, in case of threads, we use SvLOCK and al.
+#
+use threads::shared;
 
 # ABSTRACT: GNU Regular Expression Engine
 
@@ -12,7 +16,7 @@ our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 our @ISA = 'Regexp';
 
 BEGIN {
-    our $VERSION = '0.011'; # VERSION
+    our $VERSION = '0.012'; # VERSION
     XSLoader::load __PACKAGE__, $VERSION;
 }
 
@@ -102,7 +106,7 @@ re::engine::GNU - GNU Regular Expression Engine
 
 =head1 VERSION
 
-version 0.011
+version 0.012
 
 =head1 SYNOPSIS
 
