@@ -1,5 +1,5 @@
 /* Extended regular expression matching and search library.
-   Copyright (C) 2002-2015 Free Software Foundation, Inc.
+   Copyright (C) 2002-2014 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Contributed by Isamu Hasegawa <isamu@yamato.ibm.com>.
 
@@ -20,39 +20,75 @@
 #ifndef _REGEX_INTERNAL_H
 #define _REGEX_INTERNAL_H 1
 
-#include <assert.h>
-#include <ctype.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#ifdef HAVE_ASSERT_H
+#  include <assert.h>
+#endif
+#ifdef HAVE_CTYPE_H
+#  include <ctype.h>
+#endif
+#ifdef HAVE_STDIO_H
+#  include <stdio.h>
+#endif
+#ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
+#  include <string.h>
+#endif
 
+/* We do not want to include locale stuff: everything */
+/* will be done using perl API */
 #if 0
-#include <langinfo.h>
-#include <locale.h>
+#  include <langinfo.h>
+#  include <locale.h>
 #endif
-#include <wchar.h>
-#include <wctype.h>
-#ifdef HAVE_STDBOOL_H
-#include <stdbool.h>
+#ifdef HAVE_WCHAR_H
+#  include <wchar.h>
+#endif
+#ifdef HAVE_WCTYPE_H
+#  include <wctype.h>
+#endif
+#ifndef HAS_BOOL
+/* Because perl usually already defined it */
+#  ifdef HAVE_STDBOOL_H
+#    include <stdbool.h>
+#  else
+#    ifndef __cplusplus
+#      ifndef bool
+#        ifdef HAVE__BOOL
+typedef _Bool bool;
+#        else
+typedef unsigned char bool;
+#        endif
+#      endif
+#      ifndef true
+#        define true 1
+#      endif
+#      ifndef false
+#        define false 0
+#      endif
+#      define __bool_true_false_are_defined 1
+#    endif
+#  endif
 #else
-  #ifndef __cplusplus
-    #ifndef bool
-      #ifdef HAVE__BOOL
-        typedef _Bool bool;
-      #else
-        typedef unsigned char bool;
-      #endif
-    #endif
-    #ifndef true
-      #define true 1
-    #endif
-    #ifndef false
-      #define false 0
-    #endif
-    #define __bool_true_false_are_defined 1
-  #endif
-#endif
+/* This is perl's bool style. Though it usually does not define true or false */
+#  ifndef __bool_true_false_are_defined
+#    ifndef true
+#      define true 1
+#    endif
+#    ifndef false
+#      define false 0
+#    endif
+#    define __bool_true_false_are_defined 1
+#  endif
+#endif /* HAS_BOOL */
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif
+#ifdef HAVE_SYS_INT_TYPES_H
+/* for some old solaris */
+#include <sys/int_types.h>
+#endif
 
 #ifdef _LIBC
 # include <bits/libc-lock.h>
@@ -61,7 +97,7 @@
 # define lock_fini(lock) 0
 # define lock_lock(lock) __libc_lock_lock (lock)
 # define lock_unlock(lock) __libc_lock_unlock (lock)
-#elif defined GNULIB_LOCK && !defined USE_UNLOCKED_IO
+#elif defined GNULIB_LOCK
 # include "glthread/lock.h"
   /* Use gl_lock_define if empty macro arguments are known to work.
      Otherwise, fall back on less-portable substitutes.  */
@@ -83,7 +119,7 @@
 # define lock_fini(lock) glthread_lock_destroy (&(lock))
 # define lock_lock(lock) glthread_lock_lock (&(lock))
 # define lock_unlock(lock) glthread_lock_unlock (&(lock))
-#elif defined GNULIB_PTHREAD && !defined USE_UNLOCKED_IO
+#elif defined GNULIB_PTHREAD
 # include <pthread.h>
 # define lock_define(name) pthread_mutex_t name;
 # define lock_init(lock) pthread_mutex_init (&(lock), 0)
@@ -91,12 +127,12 @@
 # define lock_lock(lock) pthread_mutex_lock (&(lock))
 # define lock_unlock(lock) pthread_mutex_unlock (&(lock))
 #else
-# define lock_define(name)
-# define lock_init(lock) 0
-# define lock_fini(lock) ((void) 0)
-  /* The 'dfa' avoids an "unused variable 'dfa'" warning from GCC.  */
-# define lock_lock(lock) ((void) dfa)
-# define lock_unlock(lock) ((void) 0)
+# define lock_define(name) SV *name;
+/* GNU regex expect lock_init(lock) to return 0 if success */
+# define lock_init(lock) (SvREFCNT_inc(lock), SvSHARE(lock), 0)
+# define lock_fini(lock) SvREFCNT_dec(lock)
+# define lock_lock(lock) SvLOCK(lock)
+# define lock_unlock(lock) SvUNLOCK(lock)
 #endif
 
 /* In case that the system doesn't have isblank().  */
@@ -471,17 +507,17 @@ typedef struct re_dfa_t re_dfa_t;
 #endif
 
 #ifndef NOT_IN_libc
-static reg_errcode_t re_string_realloc_buffers (re_string_t *pstr,
+static reg_errcode_t re_string_realloc_buffers (pTHX_ re_string_t *pstr,
 						Idx new_buf_len)
      internal_function;
 # ifdef RE_ENABLE_I18N
-static void build_wcs_buffer (re_string_t *pstr) internal_function;
-static reg_errcode_t build_wcs_upper_buffer (re_string_t *pstr)
+static void build_wcs_buffer (pTHX_ re_string_t *pstr) internal_function;
+static reg_errcode_t build_wcs_upper_buffer (pTHX_ re_string_t *pstr)
   internal_function;
 # endif /* RE_ENABLE_I18N */
-static void build_upper_buffer (re_string_t *pstr) internal_function;
-static void re_string_translate_buffer (re_string_t *pstr) internal_function;
-static unsigned int re_string_context_at (const re_string_t *input, Idx idx,
+static void build_upper_buffer (pTHX_ re_string_t *pstr) internal_function;
+static void re_string_translate_buffer (pTHX_ re_string_t *pstr) internal_function;
+static unsigned int re_string_context_at (pTHX_ const re_string_t *input, Idx idx,
 					  int eflags)
      internal_function __attribute__ ((pure));
 #endif
@@ -534,9 +570,19 @@ static unsigned int re_string_context_at (const re_string_t *input, Idx idx,
 # define MIN(a,b) ((a) < (b) ? (a) : (b))
 #endif
 
-#define re_malloc(t,n) ((t *) malloc ((n) * sizeof (t)))
-#define re_realloc(p,t,n) ((t *) realloc (p, (n) * sizeof (t)))
-#define re_free(p) free (p)
+/*  t* p = malloc(n) <==> Newx(p, n, t) */
+/* #define re_malloc(t,n) ((t *) malloc ((n) * sizeof (t))) */
+#define re_malloc(dst,t,n) Newx(dst, (n) * sizeof (t), t)
+
+/* p = ((t *) realloc(p, n)) <==> Renew(p, n, t) */
+/* #define re_realloc(p,t,n) ((t *) realloc (p, (n) * sizeof (t))) */
+#define re_realloc(p,t,n) Renew(p, (n) * sizeof (t), t)
+
+#define re_calloc(dst,t,n) Newxz(dst, (n) * sizeof (t), t)
+
+/*  free(p) <==> Safefree(p) */
+/* #define re_free(p) free (p) */
+#define re_free(p) Safefree (p)
 
 struct bin_tree_t
 {
@@ -760,7 +806,10 @@ struct re_dfa_t
   lock_define (lock)
 };
 
-#define re_node_set_init_empty(set) memset (set, '\0', sizeof (re_node_set))
+/*  memset(dst, 0, n * sizeof(t)) <==> Zero(dst, n, t) */
+/* #define re_node_set_init_empty(set) memset (set, '\0', sizeof (re_node_set)) */
+#define re_node_set_init_empty(set) Zero(set, 1, re_node_set)
+
 #define re_node_set_remove(set,id) \
   (re_node_set_remove_at (set, re_node_set_contains (set, id) - 1))
 #define re_node_set_empty(p) ((p)->nelem = 0)
@@ -791,7 +840,7 @@ typedef struct
 /* Functions for bitset_t operation.  */
 
 static void
-bitset_set (bitset_t set, Idx i)
+bitset_set (pTHX_ bitset_t set, Idx i)
 {
   set[i / BITSET_WORD_BITS] |= (bitset_word_t) 1 << i % BITSET_WORD_BITS;
 }
@@ -803,19 +852,19 @@ bitset_clear (bitset_t set, Idx i)
 }
 
 static bool
-bitset_contain (const bitset_t set, Idx i)
+bitset_contain (pTHX_ const bitset_t set, Idx i)
 {
   return (set[i / BITSET_WORD_BITS] >> i % BITSET_WORD_BITS) & 1;
 }
 
 static void
-bitset_empty (bitset_t set)
+bitset_empty (pTHX_ bitset_t set)
 {
-  memset (set, '\0', sizeof (bitset_t));
+  Zero(set, 1, bitset_t);
 }
 
 static void
-bitset_set_all (bitset_t set)
+bitset_set_all (pTHX_ bitset_t set)
 {
   memset (set, -1, sizeof (bitset_word_t) * (SBC_MAX / BITSET_WORD_BITS));
   if (SBC_MAX % BITSET_WORD_BITS != 0)
@@ -824,13 +873,13 @@ bitset_set_all (bitset_t set)
 }
 
 static void
-bitset_copy (bitset_t dest, const bitset_t src)
+bitset_copy (pTHX_ bitset_t dest, const bitset_t src)
 {
-  memcpy (dest, src, sizeof (bitset_t));
+  Copy (src, dest, 1, bitset_t);
 }
 
 static void __attribute__ ((unused))
-bitset_not (bitset_t set)
+bitset_not (pTHX_ bitset_t set)
 {
   int bitset_i;
   for (bitset_i = 0; bitset_i < SBC_MAX / BITSET_WORD_BITS; ++bitset_i)
@@ -842,7 +891,7 @@ bitset_not (bitset_t set)
 }
 
 static void __attribute__ ((unused))
-bitset_merge (bitset_t dest, const bitset_t src)
+bitset_merge (pTHX_ bitset_t dest, const bitset_t src)
 {
   int bitset_i;
   for (bitset_i = 0; bitset_i < BITSET_WORDS; ++bitset_i)
@@ -850,7 +899,7 @@ bitset_merge (bitset_t dest, const bitset_t src)
 }
 
 static void __attribute__ ((unused))
-bitset_mask (bitset_t dest, const bitset_t src)
+bitset_mask (pTHX_ bitset_t dest, const bitset_t src)
 {
   int bitset_i;
   for (bitset_i = 0; bitset_i < BITSET_WORDS; ++bitset_i)
@@ -861,7 +910,7 @@ bitset_mask (bitset_t dest, const bitset_t src)
 /* Functions for re_string.  */
 static int
 internal_function __attribute__ ((pure, unused))
-re_string_char_size_at (const re_string_t *pstr, Idx idx)
+re_string_char_size_at (pTHX_ const re_string_t *pstr, Idx idx)
 {
   int byte_idx;
   if (pstr->mb_cur_max == 1)
@@ -874,7 +923,7 @@ re_string_char_size_at (const re_string_t *pstr, Idx idx)
 
 static wint_t
 internal_function __attribute__ ((pure, unused))
-re_string_wchar_at (const re_string_t *pstr, Idx idx)
+re_string_wchar_at (pTHX_ const re_string_t *pstr, Idx idx)
 {
   if (pstr->mb_cur_max == 1)
     return (wint_t) pstr->mbs[idx];
@@ -884,7 +933,7 @@ re_string_wchar_at (const re_string_t *pstr, Idx idx)
 # ifndef NOT_IN_libc
 static int
 internal_function __attribute__ ((pure, unused))
-re_string_elem_size_at (const re_string_t *pstr, Idx idx)
+re_string_elem_size_at (pTHX_ const re_string_t *pstr, Idx idx)
 {
 #  ifdef _LIBC
   const unsigned char *p, *extra;
